local RogueDefib_FrameMain;

local RogueDefib_DeadlyPoisonButton = "DEADLYPOISON";
local RogueDefib_DeadlyPoisonCounterButton = "DEADLYPOISONCOUNTER";
local RogueDefib_EnergyButton = "ENERGY";
local RogueDefib_SliceAndDiceButton = "SLICEANDDICE";
local RogueDefib_EnvenomButton = "ENVENOM";
local RogueDefib_RuptureButton = "RUPTURE";
local RogueDefib_GarroteButton = "GARROTE";

local TEXTURE_DEADLYPOISON = "Interface\\AddOns\\RogueDefibrillator\\DeadlyPoison.tga";
local TEXTURE_SLICEANDDICE = "Interface\\AddOns\\RogueDefibrillator\\SliceAndDice.tga";
local TEXTURE_GARROTE = "Interface\\AddOns\\RogueDefibrillator\\Garrote.tga";
local TEXTURE_RUPTURE = "Interface\\AddOns\\RogueDefibrillator\\Rupture.tga";
local TEXTURE_ENVENOM = "Interface\\AddOns\\RogueDefibrillator\\Envenom.tga";
local TEXTURE_COMBO_POINT = "Interface\\AddOns\\RogueDefibrillator\\ComboPoint.tga";

local RogueDefib_FrameMainButtons = {};
local RogueDefib_FrameMainButtonsOrder = {RogueDefib_DeadlyPoisonButton, RogueDefib_EnvenomButton, RogueDefib_GarroteButton, RogueDefib_RuptureButton, RogueDefib_SliceAndDiceButton};
local RogueDefib_FrameMainButtonsTextureOrder = {TEXTURE_DEADLYPOISON, TEXTURE_ENVENOM, TEXTURE_GARROTE, TEXTURE_RUPTURE, TEXTURE_SLICEANDDICE};

local RogueDefib_OnDragStart = function(self) self:StartMoving() end;
local RogueDefib_OnDragStop = function(self) self:StopMovingOrSizing() end;

local spellDeadlyPoison = 2818;
local spellSliceAndDice = 5171;
local spellGarrote = 703;
local spellRupture = 1943;
local spellEnvenom = 32645;

local SET_EXP_TIME = "expTime";
local SET_COUNT = "count";

local inCombat = false;
local enemyTarget = false;
local showMainFrame = false;

function RogueDefib_SetBackdrop(frame)
	local backdrop;
	
	if (frame:IsMouseEnabled()) then
		backdrop = {
			bgFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Background",  
			edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
			tile = true,
			tileSize = 32,
			edgeSize = 32,
			insets = {left = 11,right = 12,top = 12,bottom = 11}
		}
		frame:SetBackdrop(backdrop);
	else
		backdrop = {
			bgFile = nil,  
			edgeFile = nil,
			tile = true,
			tileSize = 32,
			edgeSize = 32,
			insets = {left = 11,right = 12,top = 12,bottom = 11}
		}
		frame:SetBackdrop(backdrop);
	end
end

function RogueDefib_CommandHandler(msg)
	if (msg == "unlock") then
		RogueDefib_FrameMain:EnableMouse(true);
		RogueDefib_SetBackdrop(RogueDefib_FrameMain);
	end
	if (msg == "lock") then
		RogueDefib_FrameMain:EnableMouse(false);
		RogueDefib_SetBackdrop(RogueDefib_FrameMain);
	end
end

function RogueDefib_OnClick()
end

function RogueDefib_OnUpdate(self)
	if (showMainFrame) then
		UpdateEnergy();
		UpdateBuffs();
		UpdateDebuffs();
	end
end

function RogueDefib_FormatTime(time)
	if (time >= 60) then
		return ceil(time / 60).."m";
	elseif(time >= 1) then
		return floor(time);
	else
		return "."..floor(time*10);
	end
end

function UpdateComboPoints()
	local comboPoints;
	comboPoints = GetComboPoints("player", "target");
	
	for i = 1,5 do
		RogueDefib_FrameMainButtons[RogueDefib_FrameMainButtonsOrder[i]].cpframe:Hide();
	end
	if (comboPoints > 0) then
		for i = 1,comboPoints do
			RogueDefib_FrameMainButtons[RogueDefib_FrameMainButtonsOrder[i]].cpframe:Show();
		end
	end
end

function UpdateEnergy()
	local power, powerMax, green;
	local threshold = 0.68;
	local button;
		
	local button = RogueDefib_FrameMainButtons[RogueDefib_EnergyButton];
	
	power = UnitPower("player", 3);
	powerMax = UnitPowerMax("player", 3);
	
	if (inCombat) then
		if (power/powerMax > threshold) then
			green = 1 - ((power-(powerMax*threshold)) / (powerMax-(powerMax*threshold)));
			button.text:SetTextColor(1,green,0,1);
		else
			button.text:SetTextColor(1,1,0,1);
		end
	else
		button.text:SetTextColor(1,1,0,1);
	end

	button.text:SetText(power);
end

function CheckDebuff(spell,setValue,emptyValue)
	local name, expTime, count, result;
	
	name,_,_,count,_,_,expTime = UnitDebuff("target", GetSpellInfo(spell));
	if(name) then
		if (setValue == SET_EXP_TIME) then result = expTime end
		if (setValue == SET_COUNT) then result = count end
	else
		result = emptyValue;
	end
	
	return result;
end

function CheckDebuffs()
	local name, expTime, count;

	RogueDefib_FrameMainButtons[RogueDefib_DeadlyPoisonButton].text.expTime = CheckDebuff(spellDeadlyPoison,SET_EXP_TIME,nil);
	RogueDefib_FrameMainButtons[RogueDefib_DeadlyPoisonCounterButton].text.count = CheckDebuff(spellDeadlyPoison,SET_COUNT,0);
	RogueDefib_FrameMainButtons[RogueDefib_GarroteButton].text.expTime = CheckDebuff(spellGarrote,SET_EXP_TIME,nil);
	RogueDefib_FrameMainButtons[RogueDefib_RuptureButton].text.expTime = CheckDebuff(spellRupture,SET_EXP_TIME,nil);
end

function UpdateDebuffs()
	if(RogueDefib_FrameMainButtons[RogueDefib_DeadlyPoisonButton].text.expTime) then
		RogueDefib_FrameMainButtons[RogueDefib_DeadlyPoisonButton].text:SetText(RogueDefib_FormatTime(RogueDefib_FrameMainButtons[RogueDefib_DeadlyPoisonButton].text.expTime-GetTime()));
		RogueDefib_FrameMainButtons[RogueDefib_DeadlyPoisonCounterButton].text:SetText(RogueDefib_FrameMainButtons[RogueDefib_DeadlyPoisonCounterButton].text.count);
	else
		RogueDefib_FrameMainButtons[RogueDefib_DeadlyPoisonButton].text:SetText("");
		RogueDefib_FrameMainButtons[RogueDefib_DeadlyPoisonCounterButton].text:SetText("0");
	end
	
	if(RogueDefib_FrameMainButtons[RogueDefib_GarroteButton].text.expTime) then
		RogueDefib_FrameMainButtons[RogueDefib_GarroteButton].text:SetText(RogueDefib_FormatTime(RogueDefib_FrameMainButtons[RogueDefib_GarroteButton].text.expTime-GetTime()));
	else
		RogueDefib_FrameMainButtons[RogueDefib_GarroteButton].text:SetText("");
	end

	if(RogueDefib_FrameMainButtons[RogueDefib_RuptureButton].text.expTime) then
		RogueDefib_FrameMainButtons[RogueDefib_RuptureButton].text:SetText(RogueDefib_FormatTime(RogueDefib_FrameMainButtons[RogueDefib_RuptureButton].text.expTime-GetTime()));
	else
		RogueDefib_FrameMainButtons[RogueDefib_RuptureButton].text:SetText("");
	end
end

function CheckBuff(spell,setValue,emptyValue)
	local name, expTime, count, result;
	
	name,_,_,count,_,_,expTime = UnitBuff("player", GetSpellInfo(spell));
	if(name) then
		if (setValue == SET_EXP_TIME) then result = expTime end
		if (setValue == SET_COUNT) then result = count end
	else
		result = emptyValue;
	end
	
	return result;
end

function CheckBuffs()
	RogueDefib_FrameMainButtons[RogueDefib_SliceAndDiceButton].text.expTime = CheckBuff(spellSliceAndDice,SET_EXP_TIME,nil);
	RogueDefib_FrameMainButtons[RogueDefib_EnvenomButton].text.expTime = CheckBuff(spellEnvenom,SET_EXP_TIME,nil);
end

function UpdateBuffs()
	if(RogueDefib_FrameMainButtons[RogueDefib_SliceAndDiceButton].text.expTime) then
		RogueDefib_FrameMainButtons[RogueDefib_SliceAndDiceButton].text:SetText(RogueDefib_FormatTime(RogueDefib_FrameMainButtons[RogueDefib_SliceAndDiceButton].text.expTime-GetTime()));
	else
		RogueDefib_FrameMainButtons[RogueDefib_SliceAndDiceButton].text:SetText("");
	end

	if(RogueDefib_FrameMainButtons[RogueDefib_EnvenomButton].text.expTime) then
		RogueDefib_FrameMainButtons[RogueDefib_EnvenomButton].text:SetText(RogueDefib_FormatTime(RogueDefib_FrameMainButtons[RogueDefib_EnvenomButton].text.expTime-GetTime()));
	else
		RogueDefib_FrameMainButtons[RogueDefib_EnvenomButton].text:SetText("");
	end
end

function RogueDefib_OnLoad(self)
	local _,class = UnitClass("player");
	if(class == "ROGUE") then
		self:SetScript("OnEvent", RogueDefib_OnEvent);
	else
		return;
	end
	
	SlashCmdList["RD"] = RogueDefib_CommandHandler;
	SLASH_RD1 = "/rd";

	self:RegisterEvent("UNIT_AURA");
	self:RegisterEvent("PLAYER_TARGET_CHANGED");
	self:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
	self:RegisterEvent("ADDON_LOADED");
	self:RegisterEvent("UNIT_FACTION");
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN");
	self:RegisterEvent("PLAYER_REGEN_DISABLED");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	self:RegisterEvent("UNIT_COMBO_POINTS");
		
	RogueDefib_FrameMain = CreateFrame("Frame", "FrameMain", UIParent);
	RogueDefib_FrameMain:Hide();
	RogueDefib_FrameMain:EnableMouse(false);
	RogueDefib_FrameMain:SetMovable(true);
	RogueDefib_FrameMain:SetBackdrop(backdrop);
	RogueDefib_FrameMain:SetPoint("TOP",UIParent,"CENTER",0,0);
	RogueDefib_FrameMain:SetHeight(60);
	RogueDefib_FrameMain:SetWidth(20+(7*40));
	RogueDefib_FrameMain:RegisterForDrag("LeftButton");
	RogueDefib_FrameMain:SetScript("OnDragStart", RogueDefib_OnDragStart);
	RogueDefib_FrameMain:SetScript("OnDragStop", RogueDefib_OnDragStop);
	RogueDefib_FrameMain:SetScript("OnUpdate", RogueDefib_OnUpdate);

	RogueDefib_SetBackdrop(RogueDefib_FrameMain);

	RogueDefib_FrameMainButtons[RogueDefib_DeadlyPoisonCounterButton] = RogueDefib_CreateIconButton(RogueDefib_FrameMain,RogueDefib_DeadlyPoisonCounterButton,0,40,40,24,nil);
	RogueDefib_FrameMainButtons[RogueDefib_DeadlyPoisonCounterButton].text:SetTextColor(0,0.5,0,1);
	for i, f in pairs(RogueDefib_FrameMainButtonsOrder) do 
		RogueDefib_FrameMainButtons[f] = RogueDefib_CreateIconButton(RogueDefib_FrameMain,f,i,40,40,18,RogueDefib_FrameMainButtonsTextureOrder[i]);		
	end
	RogueDefib_FrameMainButtons[RogueDefib_EnergyButton] = RogueDefib_CreateIconButton(RogueDefib_FrameMain,RogueDefib_EnergyButton,6,40,40,24,nil);
	
end

function RogueDefib_OnEvent(self, event, ...)
	arg1 = ...;

	if(UnitAffectingCombat("player")) then
		inCombat = true;
	else
		inCombat = false;
	end
	if(UnitExists("target") and UnitCanAttack("player", "target") and not UnitIsDead("target")) then
		enemyTarget = true
	else
		enemyTarget = false
	end
	if(enemyTarget) then
		showMainFrame = true;
	else
		showMainFrame = false;
	end

	if(event == "UNIT_AURA") then
		if(showMainFrame) then
			CheckBuffs();
			CheckDebuffs();
		end
	end 
	
	if(event == "UNIT_COMBO_POINTS") then
		UpdateComboPoints();
	end
	
	if(event == "SPELL_UPDATE_COOLDOWN") then
	end

	if(event == "PLAYER_TARGET_CHANGED") then
		if(showMainFrame) then
			CheckBuffs();
			CheckDebuffs();
			UpdateComboPoints();
		end
	end

	if(event == "ADDON_LOADED" and arg1 == "RogueDefibrillator") then
		print("Rogue Defibrillator Loaded! SlashCmdList '/rd' Options 'lock' and 'unlock'");
	end
	
	if (showMainFrame) then
		RogueDefib_FrameMain:Show();
	else
		RogueDefib_FrameMain:Hide();
	end
end

function RogueDefib_CreateIconButton(parent,name,positionInFrame,height,width,fontSize,iconTexture)
	local button;
	
	button = CreateFrame("Button", name, parent);
	button:ClearAllPoints();
	button:SetHeight(height);
	button:SetWidth(width);
	button:SetPoint("TOPLEFT",(positionInFrame*width)+10,-10);
	button:Show();

	button.text = button:CreateFontString(nil,"OVERLAY","GameFontNormal");
	button.text:SetAllPoints();
	button.text:SetFont("Interface\\AddOns\\RogueDefibrillator\\Bazooka.ttf",fontSize,"");
	
	button.cpframe = CreateFrame("Frame", nil, button);
	button.cpframe:SetFrameStrata("BACKGROUND");
	button.cpframe:SetPoint("CENTER");
	button.cpframe:SetWidth(height);
	button.cpframe:SetHeight(width);
	button.cpframe:Hide();

	button.cpicon = button.cpframe:CreateTexture(nil,"BACKGROUND");
	button.cpicon:SetTexture(TEXTURE_COMBO_POINT);
	button.cpicon:SetAllPoints(button.cpframe);
	button.cpicon:Show();
	
	button.icon = button:CreateTexture(nil,"ARTWORK")
	button.icon:SetAllPoints(button)
	button.icon:SetVertexColor(1,1,1,1);
	if (iconTexture) then
		button.icon:SetTexture(iconTexture)	
	end
	button.icon:Show();
		
	return button;
end